/**************************************************************/
/*                                                            */
/*   ho|ukahpCN[ht@C libapri.h           */
/*                                                            */
/*      Version:    2002-06-07                                */
/*                                                            */
/*      Copyright (c) 2002 T.Kondo/CRL All Right Reserved     */
/*                                                            */
/**************************************************************/
/*                                   */
/* ho|ukahpwb_[t@C  */
/*    AvIlvZp              */
/*         by T.Kondo 2002.6.7       */
/*                    2002.5.26      */
/*                                   */
#ifndef _LIBAPRI_
#define _LIBAPRI_

#include <math.h>

#ifndef CONVA
#define CONVA 4.848136811E-6    // PI*3600/180 arcsec to radian
#endif
#ifndef CONVS
#define CONVS 7.272205216E-5    // PI/(3600*12) time sec to radian
#endif
#ifndef CONVD
#define CONVD 1.745329252E-2    // PI/180  degree to radian
#endif
#ifndef CONVH
#define CONVH 2.617993878E-1    // PI/12 hour to radian
#endif
#ifndef PIRA
#define PIRA  1.745329252E-2    // PI/180  degree to radian
#endif
#ifndef PIDE
#define PIDE  57.29577951       // 180/PI radian to degree
#endif
#ifndef TWOPI
#define TWOPI 6.283185307       // 2PI
#endif
#ifndef PI
#define PI    3.141592653
#endif
#ifndef VLIGHT
#define VLIGHT 2.99792458E+8    // speed of light (m/sec)
#endif

#ifndef MAXSTAR
#define MAXSTAR 1500            // Maximum star numbers in schedule file
#endif

#ifndef MAXOBS
#define MAXOBS 2000            // Maximum observation (scan) numbers is schedule file
#endif

#ifndef FILEMAXSIZE
#define FILEMAXSIZE  2000000000   //  File maximum limit size in bytes
#endif

void mjd2ut(int *year, int *month, int *day, int *hour,
           int *min, int *sec, long j, double d);
long ut2mjd(double *d, int year, int month, int day,
           int hour, int min, int sec);

void diurnal_motion(double T0, double Ut1, double D_phai,
			double Epsiron, double *Gast, double Org[][3]);
void nutation(double t, double *D_phai, double *epsiron0,
              double *sunlon, double Org[][3]);
void precession(double t, double Org[][3]);
void rotation(double theta, double org[][3], char* axis);
void wobble(double wobx, double woby, double Org[][3]);
void cal_apri(double Time, double *Base_v_c, double *Star_v, 
			  double *Latit, double *Longi, double *Hight,
              double Rar, double Decr, double Xwob, double Ywob,
              double Ut1_c, double Offset, double Rate,
              double Pr_time,
			  double *Xtau, double *Xaz, double *Xel,
              int *Ight, double *Xuv);
void m_mat33(double A[][3], double B[][3], double C[][3]) ;
void m_mat31(double A[][3], double *B, double *C); 
void Atm(double Hight, double El, double *Xatm);
void Uv_cal(double *T_base, double Rar, double Decr, 
			 double *Xuv);
void Xyz_to_llh(double *Xyz, double *Longi, 
				double *Latit, double *Hight);
void Az_el(double Longi, double Latit, double T[][3], 
		   double *Star_v, double *Az, double *El);

typedef struct apriinfo
{
     char star[11];     // star name
     double ra;         // star position RA  in degree
     double dec;        //       "       DEC in degree
     double epo;        //       "       Epoch

     char stations[2][11];  // X,Y station name
     double xyz[2][3];     // X,Y station position (x,y,z) in m

	 double coffset;    // Clock offset at PRT  sec
	 double crate;      // Clock rate at PRT    s/s
	 double ut1_c;
	 double wobbx;
	 double wobby;

	 int    year;       // PRT year
	 int    month;      // PRT month
	 int    mday;       // PRT day of month
	 int    ddd;        // PRT day of year
	 int    hh;         // PRT hour
	 int    mm;         // PRT minutes
	 int    ss;         // PRT sec

	 double prt;        // PRT in mjd in seconds
	 int dura;       // obs duration in sec   
	 double tauap[4];   // apriori at PRT
	 int ight[5];       // greenich siderial time at PRT

     double lon[2];     // X,Y station longitude in deg
     double lat[2];     // X,Y station latitude in deg
     double h[2];     // X,Y station hight in m
	 double az[2];     // star direction (Azimuth) in deg at PRT
	 double el[2];     // star direction (Elevation) in deg at PRT

 } apri_t;
void apriori(apri_t *apri);


/* UTC珀EXւ̕ϊ */
long ut2mjd(double *d, int year, int month, int day,
           int hour, int min, int sec)
/*
	 ut2mjd   ֐l@EX
     d      iójP̎Ԃ 1 ƂƂ̐߂̎
     year   íjN
     month  íj (1 ` 12)
     day    íj
     hour   íj (0 ` 23)
     min    íj (0 ` 59)
     sec    íjb (0 ` 59)
*/
{
    int  bc, gregory, mnth, y4; 
    long j;

	mnth=month;
	y4=year;
    bc = year <= 0;
    
    /* First day at Gregory's calendar */
    gregory = year > 1582 || (year == 1582 && month > 10) ||
              (year == 1582 && month == 10 && day >= 15);
            
    if (mnth <= 2) {
        y4--;
        mnth += 12;
    }
    
    if (hour < 12) {
        j = 0;
        *d = 0.5;
    } else {
        j = 1;
        *d = -0.5;
    }
    *d += (hour*3600L + min*60 + sec)/86400.0;
    
    j += bc ? (y4-3)/4 : y4/4;
    if (gregory) j+= 2 - y4/100 + y4/400;
    j += 1720994L + y4*365L + (mnth+1)*30 + (mnth+1)*3/5 + day;
    
	/* convert to MJD */
	if (hour < 12) {
		*d -= 0.5;
	} else {
		*d += 0.5;
		j -= 1L;
	}
    j -= 2400000L;
    return j;
}

/* EXtsbɕϊ@*/
void mjd2ut(int *year, int *month, int *day,
		   int *hour, int *min, int *sec, long j, double d)
/*
     year   iójN
     month  iój
     day    iój
     hour   iój (0 ` 23)
     min    iój (0 ` 59)
     sec    iójb (0 ` 59)
     j      íjEX̐
     d      íjEX̏
*/
{
    long c, k, e, s;

    /* convert to JD */
    if( d >= 0.5) {
		d -= 0.5;
		j++;
	} else {
		d += 0.5;
	}
	j += 2400000L;

	/* JD to UTC */
    if (d >= 0.5) {
        j++;
        d -= 0.5;
    } else d += 0.5;

    if (j >= 2299161) j = j + 1 + (j-1867216)/36524 - (j-1867216)/146096;
    j += 1524;

    c = (long)((j - 122.1)/365.25);
    k = 365L*c + c/4;
    e = (long)((j - k)/30.6001);

    *year = c - 4716;
    *month = e - 1;
    if (*month > 12) {
       (*month) -= 12;
       (*year)++;
    }
    *day = j - k - (long)(30.6*e);

    s = (long)(d * 86400 + 0.5);
    *hour = s / 3600;
    *min = (s % 3600) / 60;
    *sec = s % 60;
}


void nutation(double t, double *D_phai, double *epsiron0,
              double *sunlon, double Org[][3])
{
    static int n_tab[13][7]=                      /* Nutation Table */
               {{-171996,92025, 0, 0, 0, 0, 1},
                { -13187, 5736, 0, 0, 2,-2, 2},
                {  -2274,  977, 0, 0, 2, 0, 2},
                {   2062, -895, 0, 0, 0, 0, 2},
                {   1426,   54, 0, 1, 0, 0, 0},
                {    712,   -7, 1, 0, 0, 0, 0},
                {   -517,  224, 0, 1, 2,-2, 2},
                {   -386,  200, 0, 0, 2, 0, 1},
                {   -301,  129, 1, 0, 2, 0, 2},
                {    217,  -95, 0,-1, 2,-2, 2},
                {   -158,   -1, 1, 0, 0,-2, 0},
                {    129,  -70, 0, 0, 2,-2, 1},
                {    123,  -53,-1, 0, 2, 0, 2}};
     int i;
	 double t2,t3,L,Ld,F,D,Omega,D_epsiron;
	 double S_omega;

	 /* V{p arcsec */
	 t2=pow(t,2);
	 t3=pow(t,3);
     L=485866.733+fmod(1325*t,1.0)*1296000+715922.633*t+31.31*t2+.064*t3;
     Ld=1287099.804+fmod(99*t,1.0)*1296000+1292581.224*t-.577*t2-.012*t3;
     F=335778.877+fmod(1342*t,1.0)*1296000+295263.137*t-13.257*t2+.011*t3;
     D=1072261.307+fmod(1236*t,1.0)*1296000+1105601.328*t-6.891*t2+.019*t3;
     Omega=450160.280-fmod(5*t,1.0)*1296000-482890.539*t+7.455*t2+.008*t3;

	 
	 /*   J.Wahr    */

     *D_phai=0.0;
     D_epsiron=0.0;
     for (i=0; i<13; i++)
	 {
        S_omega=(n_tab[i][2]*L+n_tab[i][3]*Ld+n_tab[i][4]*F
		    +n_tab[i][5]*D+n_tab[i][6]*Omega)*CONVA;
        *D_phai=*D_phai+n_tab[i][0]/1.0e4*sin(S_omega);
        D_epsiron=D_epsiron+n_tab[i][1]/1.e4*cos(S_omega);
	 }

	 *D_phai=*D_phai*CONVA;
     D_epsiron=D_epsiron*CONVA;
     *epsiron0=(8.4381448E+4-46.815*t-5.9E-4*t2+1.813E-3*t3)*CONVA;
     *sunlon=(F-D+Omega)*CONVA;
     //printf("epsiron0,sunlon %f %f\n",*epsiron0,*sunlon);
	 //printf("org before %f %f %f\n",Org[0][0],Org[0][1],Org[0][2]);
	 //printf("           %f %f %f\n",Org[1][0],Org[1][1],Org[1][2]);
	 //printf("           %f %f %f\n",Org[2][0],Org[2][1],Org[2][2]);

     rotation(((*epsiron0)+D_epsiron),Org,"X");   // X axis rotation
     rotation(*D_phai,Org,"Z");                // Z axis rotation
     rotation(-*epsiron0,Org,"X");    // X axis rotation
  
	 //printf("org after  %f %f %f\n",Org[0][0],Org[0][1],Org[0][2]);
	 //printf("           %f %f %f\n",Org[1][0],Org[1][1],Org[1][2]);
	 //printf("           %f %f %f\n",Org[2][0],Org[2][1],Org[2][2]);

}

void rotation(double theta, double Org[][3], char* axis)
{
     double Rot[3][3],Ans[3][3];
	 double S,C;
	 int i,k;
	 for (i=0; i<3; i++)
	 {
		 for(k=0; k<3; k++)
		 {
			 Rot[i][k]=0.0;
		 }
	 }
	 Rot[0][0]=1.0; Rot[1][1]=1.0; Rot[2][2]=1.0;
     //printf("Theta (in) = %f\n",theta);
	 S=sin(theta);
	 C=cos(theta);
	 //printf("C,S %f %f\n",C,S);
     if(strncmp(axis,"X",1)==0)    //                   X Axis Rotation
	 {
            //printf(" X selected\n");
		    Rot[1][1]=C;           //
            Rot[1][2]=S;           //   1   0   0
            Rot[2][1]=-S;          //   0   C   S
            Rot[2][2]=C;           //   0  -S   C
	 }
	 else if(strncmp(axis,"Y",1)==0)  //                Y Axis Rotation
	 {
            //printf(" Y selected\n");
            Rot[0][0]=C;           //
            Rot[2][0]=S;           //   C   0  -S
            Rot[0][2]=-S;          //   0   1   0
            Rot[2][2]=C;           //   S   0   C
	 }
     else if(strncmp(axis,"Z",1)==0)  //                Z Axis Rotation
	 {
            //printf(" Z selected\n");
            Rot[0][0]=C;           //
            Rot[0][1]=S;           //   C   S   0
            Rot[1][0]=-S;          //  -S   C   0
            Rot[1][1]=C;           //   0   0   1
	 }
     else if(strncmp(axis,"DX",2)==0)  //                X Axis Rotation
	 {
            //printf(" DX selected\n");
            Rot[0][0]=0;           //   0   0   0
            Rot[1][1]=-S;          //   0  -S   C
            Rot[1][2]=C;           //   0  -C  -S
            Rot[2][1]=-C; 
            Rot[2][2]=-S;
	 }
     else if(strncmp(axis,"DY",2)==0)  //                Y Axis Rotation
	 {
            //printf(" DY selected\n");
            Rot[1][1]=0;           //  -S   0  -C
            Rot[0][0]=-S;          //   0   0   0
            Rot[0][2]=-C;          //   C   0  -S
            Rot[2][0]=C;
            Rot[2][2]=-S;
	 }
     else if(strncmp(axis,"DZ",2)==0)  //                Y Axis Rotation
	 {
            //printf(" DZ selected\n");
            Rot[2][2]=0;           //  -S   C   0
            Rot[0][0]=-S;          //  -C  -S   0
            Rot[0][1]=C;           //   0   0   0
            Rot[1][0]=-C;
            Rot[1][1]=-S;
	 }

	 for (i=0; i<3; i++)
	 {
		 for(k=0; k<3; k++)
		 {
             Ans[i][k]=Rot[i][0]*(Org[0][k])+Rot[i][1]*(Org[1][k])+Rot[i][2]*(Org[2][k]);
		 }
	 }
	 for (i=0; i<3; i++)
	 {
		 for(k=0; k<3; k++)
		 {
             Org[i][k]=Ans[i][k];
		 }
	 }
}


void precession(double t, double Org[][3])
{
    /* precession matrix */
	double t2,t3, Guzai, Ieta, Theta;
	t2=pow(t,2);
	t3=pow(t,3);
	Guzai=(2306.2181*t+.30188*t2+.017998*t3)*CONVA;
	Ieta=(2306.2181*t+1.09468*t2+.018203*t3)*CONVA;
	Theta=(2004.3109*t-.42665*t2-.041833*t3)*CONVA;
	rotation(Ieta,Org,"Z");           // Z axis rotation
	rotation(-Theta,Org,"Y");         // Y axis rotation
	rotation(Guzai,Org,"Z");           // Z axis rotation
}

void wobble(double wobx, double woby, double Org[][3])
{
	/* Wobble Matrix */
        rotation(wobx,Org,"Y");            //! Y axis rotation
        rotation(-woby,Org,"X");           //! X axis rotation
}

void diurnal_motion(double T0, double Ut1, double D_phai,
					double Epsiron, double *Gast, double Org[][3])
{
/* **************************
  Convs TIME SEC ----> RAD
  Convd DEG      ----> RAD
  Conva arcsec   ----> RAD
  Convh HOUR     ----> RAD
 **************************
*/
      /* Diurnal Motion Matrix  */
         //  GMST : Greenwich Mean Sidereal Time
         //  GAST : Greenwich Apparent Sidereal Time
	    double T02, T03, Omega, Gmst_0h, Gmst;
		//printf("T0,Ut1,D_phai,Epsiron %g %g %g %g\n",T0,Ut1,D_phai,Epsiron);
        T02=pow(T0,2.0);
        T03=pow(T0,3.0);
        Omega=(1.00273790935080+5.9006E-11*T0-5.9E-15*T02)*CONVS;
        Gmst_0h=(24110.54841+8640184.812866*T0+.093104*T02-6.2E-6*T03)*CONVS;
        Gmst_0h=fmod(Gmst_0h, TWOPI);
        Gmst=Gmst_0h+Ut1*Omega;           // GMST (Time sec->RAD)
        *Gast=Gmst+D_phai*cos(Epsiron);    // GAST
        rotation(-(*Gast),Org,"Z");          // Z axis rotation
        //printf("Omega %g\n",Omega);
}

void m_mat33(double A[][3], double B[][3], double C[][3]) 
/* 3x3ŝZ */
{
	//double C[3][3];
	int i,k;
	for (i=0; i<3; i++)
	{
	   for(k=0; k<3; k++)
	   {
          C[i][k]=A[i][0]*B[0][k]+A[i][1]*B[1][k]+A[i][2]*B[2][k];
	   }
	}
}

void m_mat31(double A[][3], double *B, double *C) 
/* 3x1ŝZ */
{
	//double C[3][3];
	int i;
	for (i=0; i<3; i++)
	{
          C[i]=A[i][0]*B[0]+A[i][1]*B[1]+A[i][2]*B[2];
	}
}

void Az_el(double Longi, double Latit, double T[][3], 
		   double *Star_v, double *Az, double *El)
{
        double Tm_t[3][3], S_ans[3], R_ans[3], Rot_v[3][3];
		double d1,d2;
        int i,k;
	    for (i=0; i<3; i++)
		{
		   for(k=0; k<3; k++)
		   {
			 Rot_v[i][k]=0.0;
		   }
		}
	    Rot_v[0][0]=1.0; Rot_v[1][1]=1.0; Rot_v[2][2]=1.0;
        rotation(Longi,Rot_v,"Z");        // ! Z Axis
        rotation(-Latit,Rot_v,"Y");       //    ! Y Axis
	    for (i=0; i<3; i++)
		{
		   for(k=0; k<3; k++)
		   {
			 Tm_t[i][k]=T[k][i];
		   }
		}

	    for (i=0; i<3; i++)
		{
           R_ans[i]=Tm_t[i][0]*Star_v[0]+Tm_t[i][1]*Star_v[1]+Tm_t[i][2]*Star_v[2];
		}
	    for (i=0; i<3; i++)
		{
           S_ans[i]=Rot_v[i][0]*R_ans[0]+Rot_v[i][1]*R_ans[1]+Rot_v[i][2]*R_ans[2];
		}
        d1=S_ans[0];
        *El=asin(d1);
		d1=S_ans[1];
		d2=S_ans[2];
        *Az=atan2(d1,d2);
}

void Atm(double Hight, double El, double *Xatm)
{
/*
   !--------------------------------------------------------------
   ! **** ATM model is CHAO                                      !
   !      Zenith excess path is caluculated standard model       !
   !           Hight=0   P=1013.25 mb     15C                   !
   !           P=P0-0.115*Hight                                  !
   !           WV PRESS = 8.5                                    !
   !           ZP=0.002277*(P+37.5)=2.393-2.62E-4*Hight (m)      !
   !             =7.98E-9-8.74E-13*Hight (sec)                   !
   !-------------------------------------------------------------
*/
	double Zp;
    Zp=7.98E-9-8.74E-13*Hight;
    *Xatm=Zp/(sin(El)+0.00143/(tan(El)+0.0445));
}

void  Uv_cal(double *T_base, double Rar, double Decr, double *Xuv)
{

     Xuv[0]=-(-T_base[0]*sin(Rar)+T_base[1]*cos(Rar))*CONVA;
     Xuv[1]=-((-T_base[0]*cos(Rar)-T_base[1]*sin(Rar))*sin(Decr)
		     +T_base[2]*cos(Decr))*CONVA;
}


void cal_apri(double Time, double *Base_v_c, double *Star_v, 
			  double *Latit, double *Longi, double *Hight,
              double Rar, double Decr, double Xwob, double Ywob,
              double Ut1_c, double Offset, double Rate,
              double Pr_time,
			  double *Xtau, double *Xaz, double *Xel,
              int *Ight, double *Xuv)
/*    Original is  KROSS    
   !
   !-------------------------------------------------------------------------
   ! INPUT PARAMETERS ;
   !  Time              : Time on a priori value (mjd in sec) [sec]
   !  Base_v_c(1;3)     : Baseline vector / C      X,Y,Z [sec]
   !  Star_V(1;3)       : Source vector
   !  Latit(1;2)        : Latitude  of each station (rad)
   !  Longi(1;2)        : Longitude of each station (rad)
   !  Hight(1;2)        : Hight     of each station (rad)
   !  Rar               : Right Ascention (rad)
   !  Decr              : Declination (rad)
   !  Xwob              : Wobbling X parameter [rad]
   !  Ywob              : Wobbling Y parameter [rad]
   !  Ut1_c             : UT1-UTC [sec]
   !  Offset,Rate       : Clock parameter ([sec],[sec/sec]) at Pr_time
   !  Pr_time           : Reference time of clock (mjd in sec) (PRT) [sec]
   ! OUTPUT PARAMETERS;
   !  Xtau              : Delay (Sec)
   !  Xaz(1;2),Xel(1;2) : Az,El (rad)
   !  Ight(1;5)         : Greenwich Hour Time
   !  Xuv(1;2)          : U,V    (Fringe/arcsec)
   !-------------------------------------------------------------------------
   !
*/
/*  [T] = [Precession] [Nutation] [Diurnal motion] [Wobbling] */
{
    //double Ini[3][3];
    double Prec[3][3],Nuta[3][3],Diur[3][3],Wobb[3][3];
    double T[3][3],T_base[3],T1[3][3],T2[3][3];
    double Star_v_cr[3];
    double Atm_del[2];
	double Utc_t, Ut1, Bbk, Bbell, Omgat, Comg, Somg;
	double Utc_0, T0, Utc, D_phai, Epsiron, Sunlon, Gast;
	double Se, Ce, Sl, Cl;
	double Xvc, Yvc, Zvc, Normal;
	double Ra_cr, Ght, Lon, Lat, Hi, Az, El, Xatm;
	double d1, dw1;
	long mjd1;
	int i,k;
	/* matrix initialize */
	for (i=0; i<3; i++)
	{
		 for(k=0; k<3; k++)
		 {
			 Nuta[i][k]=0.0;
			 if (k==i) { Nuta[i][k]=1.0; }
			 Diur[i][k]=0.0;
			 if (k==i) { Diur[i][k]=1.0; }
			 Prec[i][k]=0.0;
			 if (k==i) { Prec[i][k]=1.0; }
			 Wobb[i][k]=0.0;
			 if (k==i) { Wobb[i][k]=1.0; }		 
		 }
	}
     /* time set */
	 mjd1=ut2mjd(&d1, 2000, 1, 1, 0, 0, 0);
     dw1=((double)mjd1 + d1)*86400.0;
     Utc_t=(Time-dw1)/86400.0;
     Utc_0=(int)(Utc_t) -0.5 ;                            // 2000y0.5d
     T0=Utc_0/36525.0  ;                            // (TDB-J2000.0)/CENT.
     Ut1=fmod(Utc_t, 1.0) ;                        // UT1 FROM 00:00:00UTC
     //printf("Ut1 (before) %g\n",Ut1);
     if (Ut1<0) {
		  Ut1=1.0+Ut1;
	 }
     //printf("Ut1 (middle) Ut1_c %g %g\n",Ut1,Ut1_c);
     Ut1=Ut1*86400.0+Ut1_c;
     //printf("Ut1 (after) %20.10e\n",Ut1);
     Utc=(Utc_t-0.5)/36525.0; // (UTC-J2000.0)/CENT.
     //printf("Utc_0, Utc_t %20.10e %20.10e\n",Utc_0,Utc_t);
     //printf("mjd1, d1, dw1 %d %f %f\n",mjd1,d1,dw1);
     //printf("Time %f\n",Time);
     //printf("Utc_t, Utc %20.10e %20.10e\n",Utc_t,Utc);
     //printf("T0, Ut1 %20.10e %20.10e\n",T0,Ut1);
	 /* Precession,Nutation,Diurnal motion,Wobbling */
     precession(Utc,Prec);
     nutation(Utc,&D_phai,&Epsiron,&Sunlon,Nuta);
     diurnal_motion(T0,Ut1,D_phai,Epsiron,&Gast,Diur);
     wobble(Xwob,Ywob,Wobb);
	 /*
     printf("Utc %f\n",Utc);
	 printf("Prec %f %f %f\n",Prec[0][0],Prec[0][1],Prec[0][2]);
	 printf("     %f %f %f\n",Prec[1][0],Prec[1][1],Prec[1][2]);
	 printf("     %f %f %f\n",Prec[2][0],Prec[2][1],Prec[2][2]);
	 printf("Nuta %f %f %f\n",Nuta[0][0],Nuta[0][1],Nuta[0][2]);
	 printf("     %f %f %f\n",Nuta[1][0],Nuta[1][1],Nuta[1][2]);
	 printf("     %f %f %f\n",Nuta[2][0],Nuta[2][1],Nuta[2][2]);
     */
	 /*--------- Transfer matrix  T */
     m_mat33( Prec, Nuta ,T1) ;                           // Precession*Nutation
     m_mat33( T1, Diur, T2)  ;                            // P*N*Diurnal motion
     m_mat33( T2, Wobb ,T) ;                            // MATRIX T=PNDW

     //printf("Base_v_c %f %f %f\n",Base_v_c[0],Base_v_c[1],Base_v_c[2]);
	 m_mat31( T, Base_v_c , T_base);                  // T *Baseline Vector
     //printf("T_base %f %f %f\n",T_base[0],T_base[1],T_base[2]);

	/* ***** Abraviation and normarized source vector
              Relativistic transform from TDB to TAI
              and movement of Y station during delay
    */
     Bbk=20.49525*CONVA;
     Bbell=Bbk*(0.01675104-0.00004180*(1.0+Utc));
     Omgat=(281.22083+0.000047084*(1.0+Utc))*CONVD-PI;
     Sunlon=fmod(Sunlon, TWOPI);
	 //printf("Sunlon %f\n",Sunlon);
     Comg=cos(Omgat);
     Somg=sin(Omgat);
     Se=sin(Epsiron);
     Ce=cos(Epsiron);
     Sl=sin(Sunlon);
     Cl=cos(Sunlon);
/*
   !---------------------------------------
   ! *****  Velocity of earth (30km/s)    !
   !---------------------------------------
*/
     Xvc=Bbk*Sl-Bbell*Somg;
     Yvc=-Bbk*Cl*Ce+Bbell*Comg*Ce;
     Zvc=-Bbk*Cl*Se+Bbell*Comg*Se;
     //printf("Xvc,Yvc,Zvc %g %g %g\n",Xvc,Yvc,Zvc);
	 //printf("Star_v  %g %g %g\n",Star_v[0],Star_v[1],Star_v[2]);
     Star_v_cr[0]=Star_v[0]+Xvc;
     Star_v_cr[1]=Star_v[1]+Yvc;
     Star_v_cr[2]=Star_v[2]+Zvc;

     Normal=sqrt(pow(Star_v_cr[0],2)+
		         pow(Star_v_cr[1],2)+
				 pow(Star_v_cr[2],2));
	for (i=0; i<3; i++)
	{
       Star_v_cr[i]= Star_v_cr[i]/Normal;   // Normalized
	}
	 //printf("Star_v_cr  %g %g %g\n",Star_v_cr[0],Star_v_cr[1],Star_v_cr[2]);

     Ra_cr=atan2(Star_v_cr[1],Star_v_cr[0]) ; //Correct Right Ascention
     //printf("Ra_cr %f\n",Ra_cr);
/*
  !------------------------------------------------
  !  Greenwich hour angle   Ight  HH,MM,SS,MSS,0  !
  !------------------------------------------------
  !
*/
     Ght=Gast-Ra_cr;
     Ght=fmod(Ght, TWOPI);
     if (Ght < 0.0) { Ght=Ght+TWOPI; }
     Ght=Ght/CONVH;
     Ight[0]=(int)Ght;
     Ght=fmod(Ght,1.0)*60.0;
     Ight[1]=(int)Ght;
     Ght=fmod(Ght, 1.0)*60.0;
     Ight[2]=(int)Ght;
     Ight[3]=(int)(fmod(Ght,1.0)*1000.0);
     Ight[4]=0;
	 //printf("Gast,Ght %g %g\n",Gast,Ght);
/*
   !
   !--------------------
   !    Az,El          !
   !--------------------
   !
*/
	 for (i=0; i<2; i++)
	 {
          Lon=Longi[i];             //  Longitude
          Lat=Latit[i];             //  Latitude
          Hi=Hight[i];              //  Hight
          Az_el(Lon,Lat,T,Star_v_cr,&Az,&El); // Az,El
          Atm(Hi,El,&Xatm);         // Atmosphere Correction
          Xaz[i]=Az;
          Xel[i]=El;
          Atm_del[i]=Xatm;
		  //printf("Lon,Lat,Hi %g %g %g\n",Lon,Lat,Hi);
		  //printf("AZ,El,Xatm %g %g %g\n",Az,El,Xatm);
	 }

/*
   !
   !----------------------
   !      Delay          !
   !----------------------
   !
*/
  /* **** Geometric DELAY  (BS)/C */
     *Xtau=Star_v_cr[0]*T_base[0]+Star_v_cr[1]*T_base[1]
		    +Star_v_cr[2]*T_base[2];
  /* **** Correction of atm and clock */
     *Xtau += (Atm_del[1]-Atm_del[0])+Offset+Rate*(Time-Pr_time);
/*
   !-----------------
   !      U,V       !
   !-----------------
*/
     Uv_cal(T_base,Rar,Decr,Xuv);
}

      /*   Change X,Y,Z (m) to Lon,Lat,Hight */
void Xyz_to_llh(double *Xyz, double *Longi, 
				double *Latit, double *Hight)
{
	  double Xlamd, Epsp, Epsh, d1, d2, d3;
      double Rearth, Eflat, Esquar;
	  double P,H, Xn, X, Y, Z, Dx, Dy, Dz, Xm, Dp, Dh;

	  //printf("%f %f %f \n",Xyz[0],Xyz[1],Xyz[2]);
      d1=Xyz[0];
	  d2=Xyz[1];
      Xlamd=atan2(d2,d1);
      Epsp=1.0E-6;
      Epsh=1.0E-4;
	  /*
   !
   !----------------------------
   !   VLBI coordinate system  !
   !----------------------------
   !
      */
      Rearth=6378137.0;
      Eflat=1.0/298.257;
      Esquar=(2.0-Eflat)*Eflat;
	  /*
   !
   !----------------------------------------
   !      INITIAL PARAMETER CALCULATED     !
   !----------------------------------------
   !
      */
	  d3=Xyz[2];
      P=atan(d3/sqrt(d1*d1+d2*d2)*(1.0-Esquar));
      H=0.0;
      /*     Iteration      */

 Loop_llh:             //  Loop iteration
	     d1=sin(P);
         Xn=Rearth/sqrt(1.0-Esquar*pow(d1,2));
         X=(Xn+H)*cos(P)*cos(Xlamd);
         Y=(Xn+H)*cos(P)*sin(Xlamd);
         Z=(Xn*(1.0-Esquar)+H)*d1;
         Dx=Xyz[0]-X;
         Dy=Xyz[1]-Y;
         Dz=Xyz[2]-Z;
		 d2=1.0-Esquar*pow(d1,2);
		 d3=pow(d2,1.5);
         Xm=Rearth*(1.0-Esquar)/d3;
         Dp=(-(Dx*cos(Xlamd)+Dy*sin(Xlamd))*sin(P)+Dz*cos(P))/(Xm+H);
         Dh=(Dx*cos(Xlamd)+Dy*sin(Xlamd)*cos(P)+Dz*sin(P));
         if (fabs(Dp) < Epsp && fabs(Dh) < Epsh) {
			 goto L3626;
		 }
         P=P+Dp;
         H=H+Dh;
         goto Loop_llh;

L3626:
      *Latit=P;
      *Longi=Xlamd;
      *Hight=H;
}

void apriori(apri_t *apri)
{
/*
   !-------------------------------------------------------------------------
   ! INPUT PARAMETERS ;
   !  apri.year         : PRT year
   !  apri.ddd          : PRT day of year
   !  apri.hh           : PRT hour
   !  apri.mm           : PRT minute
   !  apri.ss           : PRT sec
   !  apri.dura         : observation period (sec)
   !  apri.xyz[2][3]    : X Y station corrdinates  (m)
   !  apri.ra           : Right Ascention (degree)  J2000
   !  apri.dec          : Declination (degree)      J2000
   !  apri.wobbx        : Wobbling X parameter [arcsec]
   !  apri.wobby        : Wobbling Y parameter [arcsec]
   !  apri.ut1_c        : UT1-UTC [sec]
   !  appri.coffset     : Clock offset ([sec]) at PRT
   !  aprri.crate       : Clock rate ([sec/sec]) at PRT
   ! OUTPUT PARAMETERS;
   !  apri.prt          : PRT (mjd in sec)    !  apri.tauap[4]     : Calucareted apriori Delay, dealy rate ,
   !                       tau2dot,tau3dot 
   !  apri.lon[2]       : X Y Station longitude (rad) 
   !  apri.lat[2]       : X Y Station latitude (rad) 
   !  apri.h[2]         : X Y Station hight (m) 
   !  apri.az[2]        : Source azimuth (rad)  X Y 
   !  apri.el[2]        : Source elevation (rad)  X Y 
   !  apri.ight[5]      : Greenwich Hour Time
   !  //Xuv(1;2)          : U,V    (Fringe/arcsec)
   !-------------------------------------------------------------------------
   !
*/
   double base_vec[3];
   double az[2],el[2], azprt[2], elprt[2];
   double star_vec[3], ra, dec;
   double taug,time;
   double w1,w2,w3, tau[5], step;
   double xuv[2],xyz[3];
   double tw;
   //int yyyy,ddd,hh,mm,ss;
   int ight[5], ight0[5];
   int i,k;
   long mjd;
   mjd=ut2mjd(&tw,apri->year,1,1,apri->hh,apri->mm,apri->ss);
   mjd += apri->ddd-1;
   //yyyy=apri->year; ddd=apri->ddd;
   //hh=apri->hh; mm=apri->mm; ss=apri->ss;
   //mjd=ut2mjd(&tw,yyyy,1,1,hh,mm,ss);
   //mjd += ddd-1;
   apri->prt= ((double)mjd + tw)*86400.0;

   for (i=0; i<3; i++)
   {
 	  base_vec[i]=(apri->xyz[0][i]-apri->xyz[1][i])/VLIGHT;
   }
   //printf("base_vec %f %f %f\n",base_vec[0],base_vec[1],base_vec[2]);
   for (i=0; i<3; i++) {xyz[i]=apri->xyz[0][i];}
   //printf("XYZ %f %f %f\n",xyz[0],xyz[1],xyz[2]);
   Xyz_to_llh(xyz,&w1,&w2,&w3);
   apri->lon[0]=w1; apri->lat[0]=w2; apri->h[0]=w3;
   //printf("%f %f %f\n",apri->lon[0],apri->lat[0],apri->h[0]);
   for (i=0; i<3; i++) {xyz[i]=apri->xyz[1][i];}
   //printf("XYZ %f %f %f\n",xyz[0],xyz[1],xyz[2]);
   Xyz_to_llh(xyz,&w1,&w2,&w3);
	 //printf("%f %f %f \n", w1*PIDE, w2*PIDE, w3);
   apri->lon[1]=w1; apri->lat[1]=w2; apri->h[1]=w3;
   //printf("%f %f %f\n",apri->lon[1],apri->lat[1],apri->h[1]);
   ra=apri->ra*PIRA;
   dec=apri->dec*PIRA;
   //printf("ra,dec %f %f\n",apri->ra,apri->dec);
   star_vec[0]=cos(dec)*cos(ra);
   star_vec[1]=cos(dec)*sin(ra);
   star_vec[2]=sin(dec);
   //printf(" star_vec %f %f %f \n", star_vec[0],star_vec[1],star_vec[2]);
   step=apri->dura/4.0;
   time=apri->prt-2.0*step;
   //printf("PRT step %f %f\n",apri->prt,step);
   for (i=0; i<5; i++)
   {
	 
	   //printf("TIME %f\n",time);

        cal_apri(time, base_vec, star_vec, 
			  apri->lat, apri->lon, apri->h,
              ra, dec, apri->wobbx, apri->wobby,
              apri->ut1_c, apri->coffset, apri->crate,
              apri->prt,
			  &taug, az, el,
              ight, xuv);
        tau[i]=taug;
		//printf("i,taug %d %f\n",i,taug);
		if (i==2) {
			for(k=0; k<2; k++) {
				azprt[k]=az[k];
				elprt[k]=el[k];
			}
			for(k=0; k<5; k++) {
				ight0[k]=ight[k];
			}
		}
		time += step;
   }
   /*   calc tau, taudot, tau2dot, tau3dot from 5 points data */
   apri->tauap[0]=tau[2]+(12.0*(tau[1]+tau[3]-2.0*tau[2])-3.0*(tau[0]+tau[4]-2.0*tau[2]))/35.0;
   apri->tauap[1]=(tau[0]-tau[4]-8.0*(tau[1]-tau[3]))/(12.0*step);
   apri->tauap[2]=(2.0*(tau[0]-tau[2]+tau[4])-tau[1]-tau[3])/(7.0*step*step);
   apri->tauap[3]=(2.0*(tau[1]-tau[3])+tau[4]-tau[0])/(2.0*step*step*step);
   for (i=0; i<2; i++)
   {
	   apri->az[i]=azprt[i];
	   apri->el[i]=elprt[i];
   }
   for (i=0; i<5; i++)
   {
	   apri->ight[i]=ight0[i];
   }
	 //printf("taug, az, el  %g %f %f %f %f\n",tau[2],
	//	      apri->az[0]*PIDE, apri->az[1]*PIDE, apri->el[0]*PIDE, apri->el[1]*PIDE);

}

#endif
